#include "print_master_status.h"
#include "imd_monitoring.h"
#include "main.h"
#include "soc_estimation.h"
#include "stm32h7xx_hal.h"

#include "shunt_monitoring.h"
#include "ts_state_machine.h"
#include "battery.h"

#define DEBUG_CHANNEL 1 // channel to output messages on
#include "swo_log.h"

void print_master_status() {
  debug_clear_console();

  debug_log(LOG_LEVEL_INFO, "------ AMS_Master on %s (%s), compiled at %s ------", COMMIT_BRANCH,
            COMMIT_HASH, COMPILE_DATE);

  debug_log(LOG_LEVEL_INFO, "\nGeneral:");
  debug_log(LOG_LEVEL_INFO, "  State:      %s", TSStateToString(ts_state.current_state));
  debug_log(LOG_LEVEL_INFO, "  Target:     %s", TSStateToString(ts_state.target_state));
  debug_log(LOG_LEVEL_INFO, "  Err Source: %s", ts_state.error_source == 0 ? "NONE" : (ts_state.error_source == 0b11 ? "SHUNT, SLAVE" : (ts_state.error_source == 0b01 ? "SHUNT" : "SLAVE")));
  debug_log(LOG_LEVEL_INFO, "  Err type:   %s", ts_state.error_type == 0 ? "NONE" : (ts_state.error_type == 0x01 ? "SLAVE_TIMEOUT" : (ts_state.error_type == 0x02 ? "SLAVE_PANIC" : (ts_state.error_type == 0x03 ? "SHUNT_TIMEOUT" : (ts_state.error_type == 0x04 ? "SHUNT_OVERCURRENT" : (ts_state.error_type == 0x05 ? "SHUNT_OVERTEMP" : "UNKNOWN"))))));
  debug_log(LOG_LEVEL_INFO, "  HV active:  %s", hv_active ? "YES" : "NO");

  debug_log(LOG_LEVEL_INFO, "\nRelay positions:");
  debug_log(LOG_LEVEL_INFO, "  SDC:       %s", sdc_closed ? "CLOSED" : "OPEN");
  debug_log(LOG_LEVEL_INFO, "  Air-:      %s", neg_air_closed ? "CLOSED" : "OPEN");
  debug_log(LOG_LEVEL_INFO, "  Air+:      %s", pos_air_closed ? "CLOSED" : "OPEN");
  debug_log(LOG_LEVEL_INFO, "  Precharge: %s", precharge_closed ? "CLOSED" : "OPEN");
  debug_log(LOG_LEVEL_INFO, "  Precharge/Air+ sense: %s", pre_and_air_open ? "HIGH" : "LOW");

  debug_log(LOG_LEVEL_INFO, "\nIMD data:");
  debug_log(LOG_LEVEL_INFO, "  State:      %s", IMDStateToString(imd_data.state));
  debug_log(LOG_LEVEL_INFO, "  R_iso:      %lu kOhm", imd_data.r_iso);
  debug_log(LOG_LEVEL_INFO, "  Frequency:  %lu Hz", imd_data.freq);
  debug_log(LOG_LEVEL_INFO, "  Duty cycle: %lu %%", imd_data.duty_cycle);
  debug_log(LOG_LEVEL_INFO, "  Last high:  %lu ms", imd_data.last_high);
  debug_log(LOG_LEVEL_INFO, "  OK:         %s", imd_data.ok ? "YES" : "NO");

  debug_log(LOG_LEVEL_INFO, "\nShunt data:");
  debug_log(LOG_LEVEL_INFO, "  Voltage:    %ld mV", shunt_data.voltage_bat);
  debug_log(LOG_LEVEL_INFO, "  Current:    %ld mA", shunt_data.current);
  debug_log(LOG_LEVEL_INFO, "  Power:      %ld W", shunt_data.power);
  debug_log(LOG_LEVEL_INFO, "  Energy:     %ld Ws", shunt_data.energy);
  debug_log(LOG_LEVEL_INFO, "  Temp:       %ld °C", shunt_data.busbartemp / 10);
  debug_log(LOG_LEVEL_INFO, "  Time delta: %lu ms", HAL_GetTick() - shunt_data.last_message);

  debug_log(LOG_LEVEL_INFO, "\nBattery data:");
  debug_log(LOG_LEVEL_INFO, "  Min/Max voltage: %d mV / %d mV", min_voltage, max_voltage);
  debug_log(LOG_LEVEL_INFO, "  Min/Max temp:    %d °C / %d °C", min_temp, max_temp);
  debug_log(LOG_LEVEL_INFO, "  SoC:            %.2f %%", current_soc);
  debug_log(LOG_LEVEL_INFO, "  Module data: Min V | Max V | Min T | Max T");
  for (size_t i = 0; i < N_BMS; i++) {
    #if USE_ANSI_ESCAPE_CODES
      #define COLOR_MIN "\033[34m" // Blue for min
      #define COLOR_MAX "\033[31m" // Red for max
      #define COLOR_RESET "\033[0m"
      debug_log(LOG_LEVEL_INFO, "  %2zu: %s%5d mV%s | %s%5d mV%s | %s%3d °C%s | %s%3d °C%s", i,
                (module_voltages[i].min == min_voltage) ? COLOR_MIN : "", (module_voltages[i].min), COLOR_RESET,
                (module_voltages[i].max == max_voltage) ? COLOR_MAX : "", (module_voltages[i].max), COLOR_RESET,
                (module_temps[i].min == min_temp) ? COLOR_MIN : "", (module_temps[i].min), COLOR_RESET,
                (module_temps[i].max == max_temp) ? COLOR_MAX : "", (module_temps[i].max), COLOR_RESET);
    #else
    debug_log(LOG_LEVEL_INFO, "  %2zu: %5d mV | %5d mV | %3d °C | %3d °C", i,
              module_voltages[i].min, module_voltages[i].max,
              module_temps[i].min, module_temps[i].max);
    #endif
  }

  debug_log(LOG_LEVEL_INFO, "\n------ Updated at %lu ------", HAL_GetTick());
}

